@@ -46,7 +46,7 @@ Version 0.39
 
 =cut
 
-our $VERSION  = "0.39";
+our $VERSION  = "0.40";
 
 =head1 SYNOPSIS
 
@@ -795,7 +795,7 @@ sub download {
 
   my $url = @{ $pageref->{imageinfo} }[0]->{url};
 
-  unless ( $url =~ /^http\:\/\// ) {
+  unless ( $url =~ /^https?\:\/\// ) {
     return $self->_error(ERR_PARAMS,'The API returned a relative path. You need to configure the url where files are stored in {config}->{files_url}')
       unless ( defined $self->{config}->{files_url} );
     $url = $self->{config}->{files_url} . $url;
@@ -862,80 +862,70 @@ sub _make_querystring {
 # gets a token for a specified parameter and sets it in the query for the call
 sub _get_set_tokens {
   my ($self, $query) = @_;
-  my ($prop, $title, $token);
-  
-  my $action = $query->{action};
 
-  if ( $action eq 'move' ) {
-    $title = $query->{from};
-  } elsif ( $action eq 'upload' ) {
-    $action = 'edit';
-    $title = $query->{filename};
-  } elsif ( $action eq 'emailuser' ) {
-    $action = 'email';
-    $title = 'User:' . $query->{target};
-  } else {
-    $title = $query->{title};
-  }
-
-  # check if we have a cached token.
-  if ( exists( $self->{config}->{tokens}->{$action} ) ) {
-    $query->{token} = $self->{config}->{tokens}->{$action};
+  # Note that although currently many of the tokens are equivalent, we cache them separately
+  # in case this was to change.
+  if ( exists( $self->{config}->{tokens}->{$query->{action}} ) ) {
+    $query->{token} = $self->{config}->{tokens}->{$query->{action}};
     return 1;
   }
 
-  # if we are doing an import, get the edit token using Main_Page as API docs suggest.
-  if ( $action eq 'import' ) {
-    # if a title is supplied use that page to get the edit token instead of Main_Page
-    if ( defined $query->{title} ) {
-      $title = $query->{title};
-    } else {
-      $title = "Main_Page";
-    }
+  # set the token actions and title depending on edit action
+  my $tokaction = $query->{action};
+  my $title = $query->{title};
+  if ( $query->{action} eq 'move' ) {
+    $title = $query->{from};
+  } elsif ( $query->{action} eq 'upload' ) {
+    $tokaction = 'edit';
+    $title = $query->{filename};
+  } elsif ( $query->{action} eq 'emailuser' ) {
+    $tokaction = 'email';
+    $title = 'User:' . $query->{target};
+  } elsif ( $query->{action} eq 'import' ) {
+    # if title was not specified get the import token using Main_Page as API docs suggest.
+    $title = "Main_Page" unless defined $query->{title};
   }
 
-  # set the properties we want to extract based on the action
-  if ( $action eq 'rollback' ) {
-    $prop = 'revisions'; 
-  } else {
-    $prop = 'info';
+  # set the properties and token name we want to extract based on the action
+  # may merge into above ifdef depending on other token implementations
+  my $prop = 'info';
+  my $token = 'intoken';
+  my $tokquery = { action => 'query', prop => $prop, $token => $tokaction, titles => $title };  
+  if ( $tokaction eq 'rollback' ) {
+    $prop = 'revisions';
+    $token = 'rvtoken';
+  } elsif ( $tokaction eq 'patrol' ) {
+    $tokquery = { action => 'query', list => 'recentchanges', rclimit => '1', rctoken => $tokaction };
   }
 
-  $token = 'intoken';
-  $token = 'rvtoken' if ( $action eq 'rollback' );
-
   my $ref;
-  if ( $action eq 'patrol' ) {
-    $ref = $self->api( { action => 'query', list => 'recentchanges', rclimit => '1', rctoken => $action } );
-  } else {
-    $ref = $self->api( { action => 'query', prop => $prop, $token => $action, titles => $title } );
-  }
+  $ref = $self->api( $tokquery );
   return undef unless $ref;
 
   my ($pageid, $pageref) = each %{ $ref->{query}->{pages} };
 
   # if the page doesn't exist and we aren't editing/creating a new page then return an error
-  if ( defined $pageref->{missing} && $action ne 'edit' && $action ne 'import' ) {
-    return $self->_error( ERR_EDIT, "Unable to $action page '$title'. Page does not exist.") 
+  if ( defined $pageref->{missing} && $tokaction ne 'edit' && $tokaction ne 'import' ) {
+    return $self->_error( ERR_EDIT, "Unable to $tokaction page '$title'. Page does not exist.") 
   }
 
-  if ( $action eq 'rollback' ) {
-    $query->{token} = @{ $pageref->{revisions} }[0]->{$action.'token'};
+  if ( $query->{action} eq 'rollback' ) {
+    $query->{token} = @{ $pageref->{revisions} }[0]->{$tokaction.'token'};
     my $lastuser = @{ $pageref->{revisions} }[0]->{user};
     $query->{user} = @{ $pageref->{revisions} }[0]->{user} unless defined $query->{user};
     return $self->_error( ERR_EDIT, "Unable to rollback edits from user '$query->{user}' for page '$title'. Last edit was made by user $lastuser" ) if ( $query->{user} ne $lastuser );
-  } elsif ( $action eq 'patrol' ) {
+  } elsif ( $tokaction eq 'patrol' ) {
     $query->{token} = @{$ref->{query}->{recentchanges}}[0]->{patroltoken};
   } else {
-    $query->{token} = $pageref->{$action.'token'};
+    $query->{token} = $pageref->{$tokaction.'token'};
   }
 
-  return $self->_error( ERR_EDIT, "Unable to get an edit token for action '$action'." ) unless ( defined $query->{token} );
+  return $self->_error( ERR_EDIT, "Unable to get an edit token for action '$tokaction'." ) unless ( defined $query->{token} );
 
   # cache the token. rollback tokens are specific for the page name and last edited user so can not be cached. Note that although currently many of the tokens
   # are equivalent, we cache them separately in case this was to change.
-  if ( $action ne 'rollback' ) {
-    $self->{config}->{tokens}->{$action} = $query->{token};
+  if ( $query->{action} ne 'rollback' ) {
+    $self->{config}->{tokens}->{$query->{action}} = $query->{token};
   }
 
   return 1;
